#!/usr/bin/env python3

"""Create an archive in Zenodo.

Description
-----------

This script creates an archive in Zenodo and uploads the assets provided as positional arguments.
The metadata is created similar to create_datacite.

If the Zenodo ID is already present in the CodeMeta file, the existing Zenodo archive is updated instead.

Usage
-----

.. argparse::
    :module: facile_rs.create_zenodo
    :func: create_parser
    :prog: create_zenodo.py

"""

import argparse
import smtplib
from pathlib import Path

from .utils import cli, settings
from .utils.http import fetch_files
from .utils.metadata import CodemetaMetadata, ZenodoMetadata
from .utils.zenodo import create_zenodo_dataset, update_zenodo_dataset, upload_zenodo_assets


def create_parser(add_help=True):
    parser = argparse.ArgumentParser(add_help=add_help)
    parser.add_argument('assets', nargs='*', default=[],
                        help='Assets to be added to the repository.')
    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--creators-location', dest='creators_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-location', dest='contributors_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--no-sort-authors', dest='sort_authors', action='store_false',
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.set_defaults(sort_authors=True)
    parser.add_argument('--zenodo-path', dest='zenodo_path',
                        help='Path to the directory where the assets are collected before upload to Zenodo.')
    parser.add_argument('--zenodo-url', dest='zenodo_url',
                        help='URL of the Zenodo service. Test environment available at https://sandbox.zenodo.org')
    parser.add_argument('--zenodo-token', dest='zenodo_token',
                        help='Zenodo personal token.')
    parser.add_argument('--smtp-server', dest='smtp_server',
                        help='SMTP server used to inform about new release. No mail sent if empty.')
    parser.add_argument('--notification-email', dest='notification_email',
                        help='Recipient address to inform about new release. No mail sent if empty.')
    parser.add_argument('--assets-token', dest='assets_token',
                        help='Private token, to be used when fetching assets')
    parser.add_argument('--assets-token-name', dest='assets_token_name',
                        help='Name of the header field for the token [default: "PRIVATE-TOKEN"]')
    parser.add_argument('--dry', action='store_true',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'CODEMETA_LOCATION',
        'ZENODO_PATH',
        'ZENODO_URL',
        'ZENODO_TOKEN'
    ])

    # setup the bag directory
    zenodo_path = Path(settings.ZENODO_PATH).expanduser()
    if zenodo_path.exists():
        parser.error(f'{zenodo_path} already exists.')
    zenodo_path.mkdir()

    # prepare Zenodo payload
    codemeta = CodemetaMetadata()
    codemeta.fetch(settings.CODEMETA_LOCATION)
    codemeta.fetch_authors(settings.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(settings.CONTRIBUTORS_LOCATIONS)
    codemeta.compute_names()
    codemeta.remove_doubles()
    if settings.SORT_AUTHORS:
        codemeta.sort_persons()

    # override name/title to include version
    codemeta.data['name'] = '{name} ({version})'.format(**codemeta.data)

    zenodo_metadata = ZenodoMetadata(codemeta.data)
    zenodo_dict = zenodo_metadata.as_dict()

    # collect assets
    fetch_files(settings.ASSETS, zenodo_path, headers={
        settings.ASSETS_TOKEN_NAME: settings.ASSETS_TOKEN
    })

    if not settings.DRY:
        # update or create Zenodo dataset
        zenodo_id = None
        if 'identifier' in codemeta.data and isinstance(codemeta.data['identifier'], list):
            for identifier in codemeta.data['identifier']:
                if identifier.get('propertyID') == 'Zenodo':
                    zenodo_id = identifier['value']

        if zenodo_id:
            print('zenodo_id:', zenodo_id)
            dataset_id = update_zenodo_dataset(settings.ZENODO_URL, zenodo_id, settings.ZENODO_TOKEN, zenodo_dict)
        else:
            print(zenodo_id, 'not found')
            dataset_id = create_zenodo_dataset(settings.ZENODO_URL, settings.ZENODO_TOKEN, zenodo_dict)

        # upload assets
        upload_zenodo_assets(settings.ZENODO_URL, dataset_id, settings.ZENODO_TOKEN, settings.ASSETS, zenodo_path)

        if settings.SMTP_SERVER and settings.NOTIFICATION_EMAIL:
            message = """\
    From: {}
    To: {}
    Subject: {}

    {}
    """.format(
        settings.NOTIFICATION_EMAIL,
        settings.NOTIFICATION_EMAIL,
        "New Zenodo release ready to publish",
        "A new Zenodo release has been uploaded by a CI pipeline.\n\n Please visit"
        f" {settings.ZENODO_URL}/uploads/{dataset_id} to publish this release.")
            server = smtplib.SMTP(settings.SMTP_SERVER)
            server.sendmail(settings.NOTIFICATION_EMAIL, settings.NOTIFICATION_EMAIL, message)
            server.quit()


def main_deprecated():
    cli.cli_call_deprecated(main)


if __name__ == "__main__":
    main_deprecated()



#!/usr/bin/env python3

"""FACILE-RS command-line tool, used to call the different scripts of the FACILE-RS project.

Description
-----------

This script is the entry point of the FACILE-RS command-line tool.
It is used to call the different scripts of the FACILE-RS project.
Use subcommands to select a platform (Zenodo, RADAR, ...) or metadata type (CFF, DataCite,...).
Use subsubcommands to select a FACILE-RS functionality.

Usage
-----

.. argparse::
    :module: facile_rs.utils.cli
    :func: create_parser
    :prog: facile-rs

Examples
--------

To generate CFF metadata from a CodeMeta metadata file:

    $ facile-rs cff create --codemeta-location codemeta.json
"""

import argparse
import sys

from .utils import cli


def main():
    # Create the command-line parser
    parser = cli.create_parser()
    args = parser.parse_args()
    # Print help if subcommand is missing
    subcommand = args.subcommand
    if subcommand is None:
        parser.print_help()
        sys.exit(1)
    # args.mode contains the name of the script to execute
    try:
        func = args.func
    except AttributeError:
        print(f"Error: missing subcommand for '{subcommand}'.")
        # Get help for given subcommand
        for action in parser._actions:
            if isinstance(action, argparse._SubParsersAction):
                subparser = action.choices[subcommand]
                subparser.print_help()
                break
        sys.exit(1)
    # Call the script with the remaining arguments
    command_str = ' '.join(sys.argv[:3])
    sys.argv = [command_str] + sys.argv[3:]
    func()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Create an empty archive in Zenodo to reserve a DOI and a Zenodo ID.

Description
-----------

This script creates an empty archive in Zenodo in order to reserve a DOI and a Zenodo ID.
Both are stored in the CodeMeta metadata file provided as input and can be later used by the script ``create_zenodo.py``
to populate the Zenodo archive.

Usage
-----

.. argparse::
    :module: facile_rs.prepare_zenodo
    :func: create_parser
    :prog: prepare_zenodo.py

"""

import argparse
from pathlib import Path

from .utils import cli, settings
from .utils.metadata import CodemetaMetadata, ZenodoMetadata
from .utils.zenodo import create_zenodo_dataset, prepare_zenodo_dataset


def create_parser(add_help=True):
    parser = argparse.ArgumentParser(add_help=add_help)
    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--zenodo-url', dest='zenodo_url',
                        help='URL of the Zenodo service. Test environment available at https://sandbox.zenodo.org')
    parser.add_argument('--zenodo-token', dest='zenodo_token',
                        help='Zenodo personal token.')
    parser.add_argument('--dry', action='store_true',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'ZENODO_URL',
        'ZENODO_TOKEN'
    ])

    if settings.CODEMETA_LOCATION:
        codemeta = CodemetaMetadata()
        codemeta.fetch(settings.CODEMETA_LOCATION)
        name = '{name} ({version}, in preparation)'.format(**codemeta.data)
    else:
        name = 'in preparation'

    zenodo_metadata = ZenodoMetadata({'name': name})
    zenodo_dict = zenodo_metadata.as_dict()

    if not settings.DRY:
        # create Zenodo dataset
        dataset_id = create_zenodo_dataset(settings.ZENODO_URL, settings.ZENODO_TOKEN, zenodo_dict)
        dataset = prepare_zenodo_dataset(settings.ZENODO_URL, dataset_id, settings.ZENODO_TOKEN)

        doi = dataset.get('metadata', {}).get('doi', {})
        doi_url = 'https://doi.org/' + doi

        # Update Codemeta file with DOI and Zenodo ID
        if settings.CODEMETA_LOCATION:
            codemeta.data['@id'] = doi_url
            doi_entry = {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': doi
            }
            zenodo_entry = {
                '@type': 'PropertyValue',
                'propertyID': 'Zenodo',
                'value': dataset_id
            }
            if 'identifier' in codemeta.data and isinstance(codemeta.data['identifier'], list):
                found_doi = False
                found_zenodo = False
                for identifier in codemeta.data['identifier']:
                    if identifier.get('propertyID') == 'DOI':
                        identifier['value'] = doi
                        found_doi = True
                    elif identifier.get('propertyID') == 'Zenodo':
                        identifier['value'] = dataset_id
                        found_zenodo = True
                if not found_doi:
                    codemeta.data['identifier'].append(doi_entry)
                if not found_zenodo:
                    codemeta.data['identifier'].append(zenodo_entry)
            else:
                codemeta.data['identifier'] = [doi_entry, zenodo_entry]

            Path(settings.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())
        else:
            print(dataset)


def main_deprecated():
    cli.cli_call_deprecated(main)


if __name__ == "__main__":
    main_deprecated()



import argparse
import inspect
import os.path
from warnings import warn

from facile_rs import (
    create_bag,
    create_bagpack,
    create_cff,
    create_datacite,
    create_radar,
    create_release,
    create_zenodo,
    prepare_radar,
    prepare_release,
    prepare_zenodo,
    run_bibtex_pipeline,
    run_docstring_pipeline,
    run_markdown_pipeline,
)


# create the top-level parser
def create_parser():
    """
    Create parsers for the facile-rs command line interface.
    The main parser has one subparser per platform (Zenodo, RADAR, ...) or metadata type (CFF, DataCite,...).
    Each of this subparser has a subparser per FACILE-RS script.

    :return: The parser object.
    """
    # Main parser
    parser = argparse.ArgumentParser(prog='facile-rs',
        description="FACILE-RS command-line tool, to perform metadata conversion and software publication" \
            "based on CodeMeta metadata.",
        epilog="Get help on a subcommand by running 'facile-rs <subcommand> -h'.")

    # Parsers for the subcommands
    subparsers = parser.add_subparsers(help='Select the target platform or metadata type.',
                                       dest='subcommand')

    # Parser for the 'release' subcommand
    parser_release = subparsers.add_parser('release', help='Perform operations on CodeMeta metadata')
    release_subparsers = parser_release.add_subparsers()
    parser_release_prepare = release_subparsers.add_parser('prepare',
                                                           help='Update CodeMeta file with the given version and date',
                                                           parents=[prepare_release.create_parser(add_help=False)],
                                                           add_help=True)
    parser_release_prepare.set_defaults(func=prepare_release.main)

    # Parser for the 'gitlab' subcommand
    parser_gitlab = subparsers.add_parser('gitlab', help='Perform operations for GitLab releases')
    gitlab_subparsers = parser_gitlab.add_subparsers()
    parser_gitlab_publish = gitlab_subparsers.add_parser('publish',
                                                        parents=[create_release.create_parser(add_help=False)],
                                                        help="Create a release on GitLab",
                                                        add_help=True)
    parser_gitlab_publish.set_defaults(func=create_release.main)

    # Parser for the 'radar' subcommand
    parser_radar = subparsers.add_parser('radar', help='Perform operations for RADAR releases')
    radar_subparsers = parser_radar.add_subparsers()
    parser_radar_prepare = radar_subparsers.add_parser('prepare',
                                                       help='Prepare a release on RADAR',
                                                       parents=[prepare_radar.create_parser(add_help=False)],
                                                       add_help=True)
    parser_radar_prepare.set_defaults(func=prepare_radar.main)
    parser_radar_upload = radar_subparsers.add_parser('upload',
                                                      help='Create a release on RADAR',
                                                      parents=[create_radar.create_parser(add_help=False)],
                                                      add_help=True)
    parser_radar_upload.set_defaults(func=create_radar.main)

    # Parser for the 'zenodo' subcommand
    parser_zenodo = subparsers.add_parser('zenodo', help='Perform operations for Zenodo releases')
    zenodo_subparsers = parser_zenodo.add_subparsers()
    parser_zenodo_prepare = zenodo_subparsers.add_parser('prepare',
                                                        help='Prepare a release on Zenodo',
                                                        parents=[prepare_zenodo.create_parser(add_help=False)],
                                                        add_help=True)
    parser_zenodo_prepare.set_defaults(func=prepare_zenodo.main)
    parser_zenodo_upload = zenodo_subparsers.add_parser('upload',
                                                        help='Create a release on Zenodo',
                                                        parents=[create_zenodo.create_parser(add_help=False)],
                                                        add_help=True)
    parser_zenodo_upload.set_defaults(func=create_zenodo.main)

    # Parser for the 'cff' subcommand
    parser_cff = subparsers.add_parser('cff', help='Generate and manage CFF metadata')
    cff_subparsers = parser_cff.add_subparsers()
    parser_cff_create = cff_subparsers.add_parser('create',
                                                  help='Create a CFF metadata file',
                                                  parents=[create_cff.create_parser(add_help=False)],
                                                  add_help=True)
    parser_cff_create.set_defaults(func=create_cff.main)

    # Parser for the 'datacite' subcommand
    parser_datacite = subparsers.add_parser('datacite', help='Generate and manage DataCite metadata')
    datacite_subparsers = parser_datacite.add_subparsers()
    parser_datacite_create = datacite_subparsers.add_parser('create',
                                                            help='Create a DataCite metadata file',
                                                            parents=[create_datacite.create_parser(add_help=False)],
                                                            add_help=True)
    parser_datacite_create.set_defaults(func=create_datacite.main)

    # Parser for the 'bag' subcommand
    parser_bag = subparsers.add_parser('bag', help='Generate and manage BagIt bags')
    bag_subparsers = parser_bag.add_subparsers()
    parser_bag_create = bag_subparsers.add_parser('create',
                                                  help='Create a BagIt bag',
                                                  parents=[create_bag.create_parser(add_help=False)],
                                                  add_help=True)
    parser_bag_create.set_defaults(func=create_bag.main)

    # Parser for the 'bagpack' subcommand
    parser_bagpack = subparsers.add_parser('bagpack',
                                           help='Generate and manage BagPack bags (BagIt with DataCite metadata)')
    bagpack_subparsers = parser_bagpack.add_subparsers()
    parser_bagpack_create = bagpack_subparsers.add_parser('create',
                                                          help='Create a BagIt bag with DataCite metadata',
                                                          parents=[create_bagpack.create_parser(add_help=False)],
                                                          add_help=True)
    parser_bagpack_create.set_defaults(func=create_bagpack.main)

    # Parser for the 'grav' subcommand
    parser_grav = subparsers.add_parser('grav', help='Perform operations for Grav CMS')
    grav_subparsers = parser_grav.add_subparsers()
    parser_grav_bibtex = grav_subparsers.add_parser('bibtex',
                                                    help='Run the BibTex conversion pipeline',
                                                    parents=[run_bibtex_pipeline.create_parser(add_help=False)],
                                                    add_help=True)
    parser_grav_bibtex.set_defaults(func=run_bibtex_pipeline.main)

    parser_grav_docstring = grav_subparsers.add_parser('docstring',
                                                         help='Run the docstring conversion pipeline',
                                                         parents=[run_docstring_pipeline.create_parser(add_help=False)],
                                                         add_help=True)
    parser_grav_docstring.set_defaults(func=run_docstring_pipeline.main)

    parser_grav_markdown = grav_subparsers.add_parser('markdown',
                                                        help='Run the Markdown conversion pipeline',
                                                        parents=[run_markdown_pipeline.create_parser(add_help=False)],
                                                        add_help=True)
    parser_grav_markdown.set_defaults(func=run_markdown_pipeline.main)
    return parser


def cli_call_deprecated(func):
    """
    Display a deprecation warning when a script is called from the command line directly,
    without using the 'facile-rs' entry point.

    :param func: The main function to call in the script.
    """
    script_name = os.path.basename(inspect.getfile(func)).removesuffix('.py')
    warn(f"Calling {script_name} directly is deprecated. Use the entry point 'facile-rs' instead.",
         stacklevel=2)
    func()



#!/usr/bin/env python3

"""Compile and copy the content of bibtex files to a Grav CMS repository.

Description
-----------

This script compiles and copies the content of bibtex files in a similar way as run_markdown_pipeline.
A CSL can be provided.
Please refer to https://git.opencarp.org/openCARP/publications for an example setup.

Usage
-----

.. argparse::
    :module: facile_rs.run_bibtex_pipeline
    :func: create_parser
    :prog: run_bibtex_pipeline.py

"""

import argparse
import logging
from pathlib import Path

import frontmatter
import pypandoc

from .utils import cli, settings
from .utils.grav import collect_pages

logger = logging.getLogger(__file__)

TEMPLATE = '''
---
nocite: '@*'
---
'''


def create_parser(add_help=True):
    parser = argparse.ArgumentParser(add_help=add_help)

    parser.add_argument('--grav-path', dest='grav_path',
                        help='Path to the grav repository directory.')
    parser.add_argument('--pipeline', dest='pipeline',
                        help='Name of the pipeline as specified in the GRAV metadata.')
    parser.add_argument('--pipeline-source', dest='pipeline_source',
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--pipeline-csl', dest='pipeline_csl',
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'GRAV_PATH',
        'PIPELINE',
        'PIPELINE_SOURCE'
    ])

    # loop over the found pages and write the content into the files
    for page_path, page, source in collect_pages(settings.GRAV_PATH, settings.PIPELINE):
        source_path = Path(settings.PIPELINE_SOURCE).expanduser() / source
        logger.debug('page_path = %s, source_path = %s', page_path, source_path)

        extra_args = [f'--bibliography={source_path}', '--citeproc', '--wrap=preserve']
        if settings.PIPELINE_CSL:
            extra_args.append(f'--csl={settings.PIPELINE_CSL}')

        page.content = pypandoc.convert_text(TEMPLATE, to='html', format='md',
                                             extra_args=extra_args)

        logger.info('writing publications to %s', page_path)
        open(page_path, 'w').write(frontmatter.dumps(page))


def main_deprecated():
    cli.cli_call_deprecated(main)


if __name__ == "__main__":
    main_deprecated()
